(in-package "ACL2")

; This goal here is to prove that if we have a formula true
; in some interpretation I, then all instances (that are fully
; interpreted by I) are also true in I.  This isn't very far along yet.
;
; Author:  William McCune (mccune@mcs.anl.gov).

(include-book "/home/mccune/acl2/first-order/interpretations-g")

(defun add-universal-quantifiers (vars f)
  (declare (xargs :guard (and (var-list vars) (wff-d f))))
  (if (atom vars)
      f
      (list 'all (car vars) (add-universal-quantifiers (cdr vars) f))))

(defun universal-closure (f)
  (declare (xargs :guard (wff-d f)))
  (add-universal-quantifiers (free-vars f) f))

(defun quantifier-free (f)
  (declare (xargs :guard (wff-d f)))
  (cond ((wfnot f) (quantifier-free (a1 f)))
	((or (wfand f)
	     (wfor f)
	     (wfimp f)
	     (wfiff f)) (and (quantifier-free (a1 f))
			     (quantifier-free (a2 f))))
	((or (wfall f)
	     (wfexists f)) nil)
	(t t)))
  
; Function interprets-atomic-formula (i af) checks if interpretation
; i fully interprets atomic formula af.

; for verifying guards:

(defthm domain-terms-in-term-list-is-true-listp
  (true-listp (domain-terms-in-term-list x)))

(defun interprets-atomic-formula (a i)
  (declare (xargs :guard (and (interp i) (wfa-d a))))
  (and (equal (vars-in-term-list (cdr a)) nil)
       (member-equal (cons (car a) (len (cdr a)))
		     (strip-cars (relations i)))
       (subsetp-equal (func-arity-in-term-list (cdr a))
		      (strip-cars (functions i)))
       (subsetp-equal (domain-terms-in-term-list (cdr a))
		      (domain i))))

; Something like the following should be true.
;
; (defthm interprets-atomic-ok
;   (implies (and (wfa-d a)
; 		(interp i)
; 		(interprets-atomic-formula a i))
; 	   (or (equal (eval-atomic-formula a i) 'true)
; 	       (equal (eval-atomic-formula a i) 'false)))
;   :rule-classes nil
;   :otf-flg t)

(include-book "/home/shumsky/acl2/unify/unify-g")

(defun apply-subst-formula (s f)
  (declare (xargs :guard (and (well-formed-subst s)
			      (wff f)
			      (quantifier-free f))))

  (cond ((or (equal f 'true) (equal f 'false)) f)
	((wfnot f) (list 'not (apply-subst-formula s (a1 f))))
	((or (wfand f)
	     (wfor f)
	     (wfimp f)
	     (wfiff f)) (list (car f) 
			      (apply-subst-formula s (a1 f))
			      (apply-subst-formula s (a2 f))))
	((or (wfall f)
	     (wfexists f)) f)

	(t (cons (car f) (apply-subst s (cdr f))))))




